/** 
 @file mode-dialog-fields.js
 
 @brief This file contains implementation of custom fields for dialog "Edit Motion Detector Mode"

 @author: Gregor

 @date: 20.06.2013
*/

/** 
 @brief Creates custom field for input fields in "Edit Motion Detector Mode" Dialog
        - ComboBox "Mode"
        - CheckBox "Switch on the light..." with ComboBox for selecting Twilight Sensor
        - CheckBox "Set an alarm ..."
        - CheckBox "Automatically switch off..." with TimeSpan field for selecting delay
        - TimeSpan field for selecting additional delay
        - Description field
        
 
 @param 
  
 @return  Object of set values for each input field
*/

Ext.define('DSS.component.RoomFields', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
                 return 300;
    },
    rowPanel:null, rowPanelMode:null, rowPanelSwitchOff:null, rowPanelSwitchAuto:null, 
    rowPanelAdditional:null, rowPanelDescription:null, rowPanelDescription:null,
    
    comboModes:null,

    checkLight:null,
    lblCheckBox1:null,
    rowField:null,
    comboSensors:null,
    
    checkAlarm:null,
    lblCheckBox2:null,
    
    checkAutoOff:null,
    lblCheckBox3:null,
    timeSpan:null,
    
    lblDescription:null, 
    lblDescriptionLong:null, 
    
    bOnLoad : true,

    constructor : function(config) {
       var me = this;
       
       var arrayModes = new Array(
                               {'modeID':1, 'modeName': _('Corridor light')},
                               {'modeID':2, 'modeName': _('Basement light')},
                               {'modeID':3, 'modeName': _('Stairway light')},
                               {'modeID':4, 'modeName': _('Outdoor light')},
                               {'modeID':5, 'modeName': _('Switch off light')},
                               {'modeID':99, 'modeName': _('Advanced mode')}
                                 );
       this.bOnLoad = true;

       var storeData=Ext.create('Ext.data.Store', {
                           fields : [  "modeID", "modeName" ],
                           proxy : {
                               type : 'memory',
                               reader : {
                                   type : 'json',
                                   root : 'items'
                               }
                           }
                       });    
       storeData.add(arrayModes);  
        
       this.comboModes  = Ext.create('Ext.form.field.ComboBox', {
                   store: storeData,           
                   queryMode: 'local',
                   displayField: 'modeName',
                   valueField: 'modeID',                    
                   fieldLabel :  _('Mode'),
                   width : 300,
                   listeners:{
                      'select': function(objCombo){
                         var nSelectedModeID = objCombo.getValue();
                         me.setControlsStates(nSelectedModeID);
                      }
                   }
               });
               
       this.rowPanelMode = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [this.comboModes ],
            layout : 'column',
            margin : '10 0 0 0',
            width : 700
        });
        
        /* Switch off - dark only */
        var arraySensors = new Array();
        arraySensors.push({"deviceID":0,  "deviceName": _("Use position of the sun")});
        
        var deviceArray=dss.staticDataModel.getAKMDeviceArray();
        if (deviceArray != null)
        { // loading only MD devices set as Motion Detector (inputType = 4)
          for (var iIndex=0;iIndex<deviceArray.length;iIndex++) {
            if (deviceArray[iIndex] != null) {
              var deviceData = deviceArray[iIndex];
              if (deviceData.binaryInputs[0].type == 4){
                arraySensors.push({"deviceID":deviceData.dSID,  "deviceName": deviceData.name});
              }
            }
          }
        }
        
        var storeData=Ext.create('Ext.data.Store', {
                            fields : [  "deviceID", "deviceName" ],
                            proxy : {
                                type : 'memory',
                                reader : {
                                    type : 'json',
                                    root : 'items'
                                }
                            }
                        });    
        storeData.add(arraySensors);  
        
       this.checkLight = Ext.create('Ext.form.field.Checkbox', {
                                      disabledCls : 'disabledRow',
                                      fieldLabel : "",
                                      align : 'middle',
                                      listeners:{
                                        'change': function(checkbox, newVal, oldVal){
                                                      me.prepareDescription();
                                                   }
                                      }                                      
                                    })       
                                    
       this.lblCheckBox1  = Ext.create('Ext.form.Label',{
                            text : _("Switch on the light if it's dark only"),
                            margin : '0 0 0 5',
                            cls : 'x-form-item-label',
                            id: 'lblCheckBox'
                            
       }); 
       
       this.lblCheckBox2  = Ext.create('Ext.form.Label',{
                            text : _("Twilight Sensor")+":",
                            margin : '0 0 0 0',
                            cls : 'x-form-item-label',
                            id: 'lblCheckBox2'
                            
       }); 
       

       this.comboSensors  = Ext.create('Ext.form.field.ComboBox', {
                   store: storeData,           
                   queryMode: 'local',
                   displayField: 'deviceName',
                   valueField: 'deviceID',                    
                   fieldLabel : '',
                   width : 200,
                   margin : '0 0 0 10',
                   listeners:{
                      'select': function(objCombo){
                        me.prepareDescription();
                      }
                   }
               });
       var oTabPanelL = Ext.create('Ext.panel.Panel', {
                          border : false,
                          defaults: {style: {float: 'left'}},
                          items : [this.checkLight, this.lblCheckBox1],
                          width:300
                        });
       var oTabPanelR = Ext.create('Ext.panel.Panel', {
                          border : false,
                          defaults: {style: {float: 'right'}},
                          items : [this.comboSensors, this.lblCheckBox2],
                          width:430
                        });
               
       this.rowPanelSwitchOff = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [oTabPanelL, oTabPanelR],
            layout : 'column',
            margin : '10 0 0 0',
            width : 730
        });
        
       // Set Alarm
       this.checkAlarm = Ext.create('Ext.form.field.Checkbox', {
                                      fieldLabel : "",
                                      align : 'middle',
                                      listeners:{
                                        'change': function(checkbox, newVal, oldVal){
                                                      me.prepareDescription();
                                                   }
                                      }                                      
                                    })       
       this.lblCheckBox2  = Ext.create('Ext.form.Label',{
            text : _("Set an alarm if motion was detected while being absent"),
            align : 'middle',
            margin : '0 0 0 5',
            cls : 'x-form-item-label',
       }); 
       
       this.rowPanelSwitchAlarm = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [this.checkAlarm, this.lblCheckBox2 ],
            layout : 'column',
            margin : '10 0 0 0',
            width : 700
        });
        
       // Auto off
       this.checkAutoOff = Ext.create('Ext.form.field.Checkbox', {
                                      fieldLabel : "",
                                      align : 'middle',
                                      listeners:{
                                        'change': function(checkbox, newVal, oldVal){
                                                      me.prepareDescription();
                                                   }
                                      }                                      
                                    });
                                    
       this.lblCheckBox3  = Ext.create('Ext.form.Label',{
            text : _("Automatically switch off the light if no motion was detected after"),
            align : 'middle',
            cls : 'x-form-item-label',
            margin : '0 0 0 5'
       }); 
       
       this.timeSpanAuto  = Ext.create('DSS.component.TimeSpanControl', {
            fieldLabel : "",
            hideSecond : true,
            margin : '0 0 0 0',
            width:200,
            listeners: {
              change: function(field, value) {
                me.prepareDescription();
              }
            }
       });
       
       var oTabPanelL = Ext.create('Ext.panel.Panel', {
                          border : false,
                          defaults: {style: {float: 'left'}},
                          items : [this.checkAutoOff, this.lblCheckBox3],
                          width:500
                        });
       var oTabPanelR = Ext.create('Ext.panel.Panel', {
                          border : false,
                          defaults: {style: {float: 'right'}},
                          items : [this.timeSpanAuto],
                          width:230
                        });
       

       this.rowPanelSwitchAuto = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [oTabPanelL,   oTabPanelR],
            layout : 'column',
            margin : '10 0 0 0',
            width : 730
        });
        
       // Additional switch off
       this.lblAdditional  = Ext.create('Ext.form.Label',{
            text : _("Additional switch-off delay after the motion detector did switch off"),
            align : 'middle',
            cls : 'x-form-item-label',
            width:530,
       }); 
       
       this.timeSpanAdditional  = Ext.create('DSS.component.TimeSpanControl', {
            hideHour : true,
            margin : '0 0 0 0',
            width:100,
            listeners: {
              change: function(field, value) {
                me.prepareDescription();
              }
            }
       });

       this.rowPanelAdditional = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [this.lblAdditional,  this.timeSpanAdditional ],
            layout : 'column',
            margin : '10 0 0 0',
            width : 750
        });
        
       this.lblDescription  = Ext.create('Ext.form.field.Display',{
            fieldLabel : _("Description"),
            align : 'middle',
            cls : 'x-form-item-label',
       }); 
 
        
       this.lblDescriptionLong = Ext.create('Ext.form.Label',{
            html : "",
            align : 'left',
            cls : 'long-desc-field',
       }); 
        
        
       this.rowPanelDescription = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [this.lblDescription,  this.lblDescriptionLong ],
            layout : 'column',
            margin : '10 0 0 0',
            width : 750
        });
        
        this.rowPanel = Ext.create('Ext.panel.Panel', {
            border : false,
            autoScroll : false,
            height : 300,
            layout : {
                type : 'vbox',
                flex:0
            },
            items : [ this.rowPanelMode, 
                      this.rowPanelSwitchOff, 
                      this.rowPanelSwitchAlarm, 
                      this.rowPanelSwitchAuto, 
                      this.rowPanelAdditional,
                      this.rowPanelDescription]
        });
        

        Ext.apply(this, {
            items : [ this.rowPanel ]
        });       
        
        this.callParent(arguments);
    },
    
    leadingZero : function (num, places) {
      var zero = places - num.toString().length + 1;
      return Array(+(zero > 0 && zero)).join("0") + num;
    },
    
    prepareTime: function(secs) {
      var hours = Math.floor(secs / (3600));
     
      var divisor_for_minutes = secs % (3600);
      var minutes = Math.floor(divisor_for_minutes / 60);     
     
      var divisor_for_seconds = divisor_for_minutes % 60;
      var seconds = Math.ceil(divisor_for_seconds);
      
      var pTime = new Object();
      pTime.hours = "00";
      pTime.minutes = "00";
      pTime.seconds = "00";
      
      if (hours > 0)
        pTime.hours = this.leadingZero(hours, 2);
      
      if (minutes > 0)
        pTime.minutes = this.leadingZero(minutes, 2);
     
      if (seconds > 0)
        pTime.seconds = this.leadingZero(seconds, 2);
      
      return pTime;
    },    
    
    prepareDescription: function() {
      if (!this.bOnLoad) {
      
        var sLabelText = "";
        var nMode = this.comboModes.getValue();

        if (nMode < 5) {
          var sSwitchOnLight = _("Switch on the light throughout the day");
          // Switch on when dark
          if (!this.checkLight.disabled && this.checkLight.checked) {
           if (this.comboSensors.getValue()==0 || this.comboSensors.getValue()==-1) // MD is selected
              sSwitchOnLight = _("Switch on the light between sunset and sunrise");
            else // Use position of the sun is selected
              sSwitchOnLight = _("Switch on the light if it's dark only");
          }
          sLabelText += "<li>"+sSwitchOnLight+"</li>";
        }

        var sAdditionalSwitchOff = "";
        var sExtraLabel = "";
        switch(nMode) {
          case 1: // Corridor Light
          case 2: // Basement Light
            sExtraLabel = "%AdditionalOff";
            sExtraLabel += "<li>"+_("Do not switch off the light if preset 2-4 was called")+"</li>";
            sExtraLabel += "<li>"+_("Dimmable light will slowly fade off within 1 minute")+"</li>";
            if(!this.checkAlarm.checked)
              sExtraLabel += "<li>"+_("'Coming home' will be called if the light turns on")+"</li>";
            
            sAdditionalSwitchOff = _("Switch off the light with a delay of %minutes:%seconds (mm:ss) if switched on automatically or preset 1 was called");
          break;
          
          case 3: // Stairway light
            sExtraLabel = "%AdditionalOff";
            sExtraLabel += "<li>"+_("Dimmable light will slowly fade off within 1 minute")+"</li>";
            if(!this.checkAlarm.checked)
              sExtraLabel += "<li>"+_("'Coming home' will not be called if the light turns on")+"</li>";
            
            sAdditionalSwitchOff = _("Switch off the light with a delay of %minutes:%seconds (mm:ss) regardless of the selected preset");
          break;
          
          case 4: // Outdoor light
            sExtraLabel += "%AdditionalOff";
            if(!this.checkAlarm.checked)
              sExtraLabel += "<li>"+_("'Coming home' will not be called if the light turns on")+"</li>";
            
            sAdditionalSwitchOff = _("Switch off the light with a delay of %minutes:%seconds (mm:ss) regardless of the selected preset");
          break;
          
          case 5: // Switch off light
            sExtraLabel = "<li>"+_("Do not switch on the light automatically")+"</li>";
            sExtraLabel += "%AdditionalOff";
            sExtraLabel += "<li>"+_("Do not switch off the light if preset 2-4 was called")+"</li>";
            sExtraLabel += "<li>"+_("Dimmable light will slowly fade off within 1 minute")+"</li>";
            
            sAdditionalSwitchOff = _("Switch off the light with a delay of %minutes:%seconds (mm:ss) if preset 1 was called");
          break;
          
          case 99: // Advanced mode
             sExtraLabel = "<li>"+_("The light does not turn on or off automatically")+"</li>";
             sExtraLabel += "<li>"+_("Use the Event-Responder app to configure a user specific behaviour based on the motion state within the room")+"</li>";
          break;
        }
        
        var sAdditionalText = "";
        if (this.timeSpanAdditional.getValue()>0) {
          var pTime = this.prepareTime(this.timeSpanAdditional.getValue());
          sAdditionalSwitchOff = sAdditionalSwitchOff.replace("%minutes", pTime.minutes).replace("%seconds", pTime.seconds);
          
          sAdditionalText = "<li>"+sAdditionalSwitchOff+"</li>";
        }
        sExtraLabel = sExtraLabel.replace("%AdditionalOff", sAdditionalText)
        
        if (sExtraLabel != "")
          sLabelText += sExtraLabel;

        // Set alarm
        if (!this.checkAlarm.disabled && this.checkAlarm.checked)
          sLabelText += "<li>"+_("Alarm 1 will be called instead of switching on the light if motion was detected during your absence")+"</li>";
                                  
        // Auto switch off
        if (!this.checkAutoOff.disabled && this.checkAutoOff.checked) {
          var pTime = this.prepareTime(this.timeSpanAuto.getValue());
          // if checked and value is set to 00:00 we set minimum value to 1 minute
          if (pTime.hours == '00' && pTime.minutes == '00') {
            pTime.minutes = '01';
            this.timeSpanAuto.setValue(60);
          }  
          
          var sInfo = _("If the light was turned on manually, it will be switched off automatically after %hours:%minutes (hh:mm)"); 
          sInfo = sInfo.replace("%hours", pTime.hours).replace("%minutes", pTime.minutes);
          sLabelText += "<li>"+sInfo+"</li>";
        }
        
       
        this.lblDescriptionLong.setText(sLabelText, false);
      }
    },
    
    setControlsStates: function(modeID) {
      switch(modeID){
        case 0:
        case 99:
          this.rowPanelSwitchOff.disable(); 
          this.rowPanelSwitchAlarm.disable(); 
          this.rowPanelSwitchAuto.disable(); 
          this.rowPanelAdditional.disable(); 

          /* Set Default values*/
          this.checkLight.setValue(false);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(false);
          this.timeSpanAuto.setValue(0);
          this.timeSpanAdditional.setValue(0);
        break;
        
        case 1: // Corridor Light
          this.rowPanelSwitchOff.enable(); 
          this.rowPanelSwitchAlarm.enable();
          this.rowPanelSwitchAuto.enable(); 
          this.rowPanelAdditional.enable(); 
          
          /* Set Default values*/
          this.checkLight.setValue(true);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(false);
          this.timeSpanAuto.setValue(0);
          this.timeSpanAdditional.setValue(0);
        break;

        case 2: // Basement Light
          this.rowPanelSwitchOff.enable(); 
          this.rowPanelSwitchAlarm.enable();
          this.rowPanelSwitchAuto.enable(); 
          this.rowPanelAdditional.enable(); 
          
          /* Set Default values*/
          this.checkLight.setValue(false);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(false);
          this.timeSpanAuto.setValue(0);
          this.timeSpanAdditional.setValue(180);
        break;

        case 3: // Stairway light
          this.rowPanelSwitchOff.enable(); 
          this.rowPanelSwitchAlarm.enable();
          this.rowPanelSwitchAuto.enable(); 
          this.rowPanelAdditional.enable(); 

          /* Set Default values*/
          this.checkLight.setValue(false);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(true);
          this.timeSpanAuto.setValue(180);
          this.timeSpanAdditional.setValue(0);
        break;

        case 4: // Outdoor light
          this.rowPanelSwitchOff.enable(); 
          this.rowPanelSwitchAlarm.disable();
          this.rowPanelSwitchAuto.enable();  
          this.rowPanelAdditional.enable(); 
          
          /* Set Default values*/
          this.checkLight.setValue(true);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(true);
          this.timeSpanAuto.setValue(3600);
          this.timeSpanAdditional.setValue(0);
        break;
        
        case 5: // Switch off light
          this.rowPanelSwitchOff.disable(); 
          this.rowPanelSwitchAlarm.disable();
          this.rowPanelSwitchAuto.enable(); 
          this.rowPanelAdditional.enable(); 

          /* Set Default values*/
          this.checkLight.setValue(false);
          this.comboSensors.setValue(0);
          this.checkAlarm.setValue(false);
          this.checkAutoOff.setValue(false);
          this.timeSpanAuto.setValue(0);
          this.timeSpanAdditional.setValue(180);
        break;
      }
      this.prepareDescription();
    }, 
    
    reset: function (){
      this.bOnLoad = true; 
     
      var nMode = 99; // default advanced mode
      if (this.dssEditor.editControls.modeID.getValue() > 0)
        nMode = this.dssEditor.editControls.modeID.getValue();
      this.comboModes.setValue(nMode);
      this.setControlsStates(this.dssEditor.editControls.modeID.getValue());
      
      this.checkLight.setValue(this.dssEditor.editControls.isOnlyDark.getValue());
      var nSelectedTwilight = this.dssEditor.editControls.onlyDarkTwilightID.getValue();
      if (nSelectedTwilight != "")
        this.comboSensors.setValue(nSelectedTwilight);

      this.checkAlarm.setValue(this.dssEditor.editControls.isSetAlarm.getValue());

      this.checkAutoOff.setValue(this.dssEditor.editControls.isAutoOff.getValue());
      this.timeSpanAuto.setValue(this.dssEditor.editControls.autoOffTime.getValue());

      this.timeSpanAdditional.setValue(this.dssEditor.editControls.additionalOffDelayTime.getValue());
        
      this.bOnLoad = false; 
      this.prepareDescription();
    },
    
    setValue:function (value){
    },
    
    isValid:function(){
        return true;
    },
    
    getValue : function(){
      var returnValue = new Object();
      returnValue.modeID     = this.comboModes.getValue();
      
      returnValue.isDark     = this.checkLight.getValue();
      returnValue.twilightID = this.comboSensors.getValue();
      
      returnValue.isAlarm    = this.checkAlarm.getValue();
      
      returnValue.isAutoOff  = this.checkAutoOff.getValue();
      returnValue.delayTime  = this.timeSpanAuto.getValue();
      
      returnValue.delayExtra = this.timeSpanAdditional.getValue();
      
      return returnValue;
    }
});

/** 
 @brief Creates custom field for showing Room Name
 @param 
 @return  -
*/
Ext.define('DSS.component.RoomLabelControl', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
                 return 20;
    },
    lblLabel:null,
    constructor : function(config) {

        this.lblLabel  = Ext.create('Ext.form.field.Display',{
          fieldLabel: _('Room'),
        }); 
        
        Ext.apply(this, {
            items : [ this.lblLabel ]
        });                          
        this.callParent(arguments);
    },
    reset: function (){
     this.lblLabel.setValue(' - ');
    },
    setValue:function (value){
      this.lblLabel.setValue(dss.staticDataModel.getNameForZone(value));
    },
    isValid:function(){
        return true;
    },
    getValue : function(){

    }
});